package de.ullisroboterseite.ursanalogclock;

// Autor: https://UllisRoboterSeite.de

// Doku:  https://UllisRoboterSeite.de/android-AI2-visible.html
// Created: 2020-12-13
//
// Version 1 (2020-12-13)
// -------------------------
// - Basis-Version

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import java.lang.reflect.*;

import android.app.Activity;
import android.os.Handler;
import android.util.Log;

import android.view.View;
import android.view.ViewGroup;
import android.widget.TableLayout;
import android.widget.TableRow;

// Zum Testen
import java.io.StringWriter;
import java.io.PrintWriter;

@SimpleObject(external = true)
public abstract class UrsViewBase extends AndroidNonvisibleComponent implements OnInitializeListener {
  static final String LOG_TAG = "VIS";
  private final Activity thisActivity;
  final ComponentContainer container; // Container für diese nicht sichtbare Komponente (ist Screen)
  final UrsViewBase thisInstance = this;
  Handler handler = new Handler();

  // Referenz auf die neue (Ersatz-) Komponente
  AndroidViewComponent newViewComponent = null;

  // Die Platzhalterkomponennte
  AndroidViewComponent placeholder = null;

  // Container-Komponete in die die Platzhalterkomponente eingebunden ist.
  ComponentContainer placeholderParent = null;

  // Das Android-Container-Object, dass den View der Paltzhaltgerkomponente verwaltet.
  android.view.ViewGroup layoutManager = null;

  boolean isTable = false; // Ob der Parant-Container ein TableArrangement ist.

  public UrsViewBase(ComponentContainer container) { // Dies ist der Screen
    super(container.$form());
    this.container = container;
    thisActivity = container.$context();
    form.registerForOnInitialize(this);
  }

  @DesignerProperty(editorType = PropertyTypeConstants.PROPERTY_TYPE_COMPONENT, defaultValue = "")
  @SimpleProperty(description = "Placeholder component", userVisible = false)
  public void PlaceHolder(AndroidViewComponent viewComponent) {
    placeholder = viewComponent;
  }

  /**
  * Erstellt die sichtbare Komponente
  * @param container Der Container, in die die Komponente eingebunden werden soll
  * @return Eine Instanz der sichtbaren Komponente
  */
  abstract AndroidViewComponent createViewComponent(ComponentContainer container, Button placeholder);

  /**
   * Wird ausgelöst, wenn die Ersetzung stattgefunden hat.
   * @param container Der Container der neu erstellten Komponente
   * @param placeholder Der Platzhalter
   * @note Width und Height werden über den Containter eingestellt und
   * kann deshalb erst nache dem Ersatz festgelegt werden.
   */
  void replacementDone(ComponentContainer container, Button placeholder) {
  }

  /**
   * Wird vom übergeordneten Form aufgerufen nachdem das Ereignis Screen.Initialize ausgelöst wurde.
   */
  @Override
  public void onInitialize() {
    // Wenn im Companion eine neue Komponente eingefügt wird,
    // ist die PlaceHolder-Komponente noch festgelegt.
    if (placeholder == null)
      return;

    // Für den Paltzhalter sind nur Komponenten vom Typ Button erlaubt.
    if (!(placeholder instanceof Button)) {
      Notifier.oneButtonAlert(thisActivity, "App will be canceled",
          "Error on Placeholder property: Invalid component type", "OK", new Runnable() {
            public void run() {
              Form.finishApplication();
            }
          });
      return;
    }

    // replacePlaceholder nicht direkt sondern verzögert ausführen.
    // Die TableArrangements sind noch nicht korrekt initialisiert.
    handler.post(new Runnable() {
      public void run() {
        replacePlaceholder();
      }
    });
  }

  /**
   * Ersetzt die Platzhalter-Komponente im Container.
   */
  void replacePlaceholder() {
    try {
      placeholderParent = Helper.getControlsParent(placeholder);
      newViewComponent = createViewComponent(placeholderParent, (Button) placeholder);
      layoutManager = Helper.getContainersViewGroup(placeholder);

      if (layoutManager instanceof android.widget.TableLayout) {
        int row = placeholder.Row();
        int col = placeholder.Column();
        TableRow tableRow = (TableRow) layoutManager.getChildAt(row);
        tableRow.removeViewAt(col);
        View cellView = newViewComponent.getView();
        tableRow.addView(cellView, col, placeholder.getView().getLayoutParams());
      } else {
        int ind = layoutManager.indexOfChild(placeholder.getView());
        layoutManager.removeView(placeholder.getView());
        layoutManager.addView(newViewComponent.getView(), ind, new android.widget.LinearLayout.LayoutParams(
            ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT, 0f));
      }

      placeholder.Visible(false); // Die Ansicht des Platzhalters wird nicht mehr benötigt.

      replacementDone(placeholderParent, (Button) placeholder);
      ComponentReplaced();

    } catch (Exception e) {
      StringWriter sw = new StringWriter();
      PrintWriter pw = new PrintWriter(sw);
      e.printStackTrace(pw);
      String stackTrace = sw.toString(); // stack trace as a string
      Log.d(LOG_TAG, "Error in replacePlaceholder: " + e.toString());
      Log.d(LOG_TAG, stackTrace);

      Notifier.oneButtonAlert(thisActivity, "App will be canceled", "Cannot replace component", "OK", new Runnable() {
        public void run() {
          Form.finishApplication();
        }
      });
      return;
    }
  }

  @SimpleEvent(description = "Fired when component was replaced.")
  public void ComponentReplaced() {
    handler.post(new Runnable() {
      public void run() {
        EventDispatcher.dispatchEvent(thisInstance, "ComponentReplaced");
      } // run
    }); // post
  }
}